const mongoose = require('mongoose')
const bcrypt = require('bcrypt')
const validator = require('validator')
const mongoosePaginate = require('mongoose-paginate-v2')
const mongooseAlgolia = require('mongoose-algolia')
const Schema = mongoose.Schema;
const UserSchema = new mongoose.Schema({
    // posts: [{ type: Schema.Types.ObjectId, ref: 'Posts' }],
    name: {
        type: String,
        required: true
    },
    country: {
        type: Object,
        default: {}
    },
    phoneNumber: {
        type: Object,
        default: { value: '', dailCode: '' }
    },
    birthday: {
        type: String,
        default: ''
    },
    gender: {
        type: String,
        enum: ['male', 'female', 'other'],
        required: false
    },
    lastName: {
        type: String,
        required: true
    },
    firstName: {
        type: String,
        required: true
    },
    userName: {
        type: String,
        lowercase: true,
        unique: true,
        required: true
    },
    email: {
        type: String,
        validate: {
            validator: validator.isEmail,
            message: 'EMAIL_IS_NOT_VALID'
        },
        lowercase: true,
        unique: true,
        required: true
    },
    password: {
        type: String,
        required: false,
        select: false
    },
    role: {
        type: String,
        enum: ['user', 'creator', 'admin'],
        default: 'user'
    },
    verification: {
        type: String
    },
    verified: {
        type: Boolean,
        default: false
    },
    isProfileComplite: {
        type: Boolean,
        default: false
    },
    address: {
        type: Object,
    },
    payoutInfo: {
        type: Object,
        default: {
            isSet: false,
            getways: []
        }
    },
    profileInfo: {
        type: Object
    },
    pictures: {
        type: Object,
        default: {
            profilePicture: 'https://res.cloudinary.com/fanband/image/upload/v1621563138/user_lbzcan.png',
            cover: 'https://res.cloudinary.com/fanband/image/upload/v1644593174/136-1365453_no-copyright-background-images-brighton_shfbab.jpg',
            profilePictureThump: 'https://res.cloudinary.com/fanband/image/upload/c_scale,q_100,w_300/v1621563138/user_lbzcan.png',

        }
    },
    addresses: {
        type: Array
    },
    loginAttempts: {
        type: Number,
        default: 0,
        select: false
    },
    fcmToken: { type: String },
    blockExpires: {
        type: Date,
        default: Date.now,
        select: false
    },
    provider: {

        type: String,
        default: 'fanband'
    },
    providerId: {
        type: String,
        default: 'fanband'
    },
    paymentInfo: {
        type: Object,
        default: {
            stripe: {
                _id: ''
            }
        },
    }
}, {
    versionKey: false,
    timestamps: true
})

const hash = (user, salt, next) => {
    bcrypt.hash(user.password, salt, (error, newHash) => {
        if (error) {
            return next(error)
        }
        user.password = newHash
        return next()
    })
}

const genSalt = (user, SALT_FACTOR, next) => {
    bcrypt.genSalt(SALT_FACTOR, (err, salt) => {
        if (err) {
            return next(err)
        }
        return hash(user, salt, next)
    })
}

UserSchema.pre('save', function(next) {
    const that = this
    const SALT_FACTOR = 5
    if (!that.isModified('password')) {
        return next()
    }
    return genSalt(that, SALT_FACTOR, next)
})

UserSchema.methods.comparePassword = function(passwordAttempt, cb) {
    bcrypt.compare(passwordAttempt, this.password, (err, isMatch) =>
        err ? cb(err) : cb(null, isMatch)
    )
}
UserSchema.plugin(mongooseAlgolia, {
    appId: 'B4WLTPFLQ0',
    apiKey: 'e50b55681b994f62792f06595a83170c',
    indexName: 'clubs', //The name of the index in Algolia, you can also pass in a function
    selector: 'name userName pictures', //You can decide which field that are getting synced to Algolia (same as selector in mongoose)
    populate: {
        path: 'userName',
        select: 'name',
    },

    virtuals: {
        whatever: function(doc) {
            return {
                name: doc.name,
                userName: doc.userName,
                profilePicture: doc.pictures.profilePictureThump,
            }
        },
    },
    filter: function(doc) {
        if (doc.role === 'user') {
            return doc.softdelete
        }

        return !doc.softdelete
    },
    // Default: false -> If true operations are logged out in your console
})

let Model = mongoose.model('User', UserSchema)

Model.SyncToAlgolia() //Clears the Algolia index for this schema and synchronizes all documents to Algolia (based on the settings defined in your plugin settings)
Model.SetAlgoliaSettings({
    searchableAttributes: ['name', 'userName'], //Sets the settings for this schema, see [Algolia's Index settings parameters](https://www.algolia.com/doc/api-client/javascript/settings#set-settings) for more info.
})
UserSchema.plugin(mongoosePaginate)
module.exports = Model