require('dotenv-safe').config({
    allowEmptyValues: true
})
const express = require('express')
const bodyParser = require('body-parser')
const morgan = require('morgan')
const compression = require('compression')
const helmet = require('helmet')
const cors = require('cors')
const passport = require('passport')
const app = express()
const i18n = require('i18n')
const initMongo = require('./config/mongo')
const path = require('path')
const bcrypt = require('bcrypt')

// Setup express server port from ENV, default: 3000
app.set('port', process.env.PORT || 3000)


// Enable only in development HTTP request logger middleware
if (process.env.NODE_ENV === 'development') {
    app.use(morgan('dev'))
}
// start admin js


const AdminJS = require('adminjs')
const AdminJSExpress = require('@adminjs/express')
const AdminJSMongoose = require('@adminjs/mongoose')
AdminJS.registerAdapter(AdminJSMongoose)
const user = require('./app/models/user')
const posts = require('./app/models/posts')
const payouts = require('./app/models/payouts')
const staff = require('./app/models/staff')
const adminJs = new AdminJS({
    resources: [user, posts, payouts, {
        resource: staff,
        options: {
            properties: {
                encryptedPassword: {
                    isVisible: false,
                },
                password: {
                    type: 'string',
                    isVisible: {
                        list: false,
                        edit: true,
                        filter: false,
                        show: false,
                    },
                },
            },
            actions: {
                new: {
                    before: async(request) => {
                        if (request.payload.password) {
                            request.payload = {
                                ...request.payload,
                                encryptedPassword: await bcrypt.hash(request.payload.password, 10),
                                password: undefined,
                            }
                        }
                        return request
                    },
                }
            }
        }
    }],
    rootPath: '/server/mng',
    logoutPath: '/server/mng/exit',
    loginPath: '/server/mng/sign-in',
    assetsPath: './public/assets/',
    branding: {
        companyName: 'fanband',
        softwareBrothers: false,
        logo: './public/assets/logo.png'
    },
})

const adminRouter = AdminJSExpress.buildAuthenticatedRouter(adminJs, {
    authenticate: async(email, password) => {
        const user = await staff.findOne({ email })
        if (user) {
            const matched = await bcrypt.compare(password, user.encryptedPassword)
            if (matched) {
                return user
            }
        }
        return false
    },
    cookiePassword: 'some-secret-password-used-to-secure-cookie',
})
app.use(adminJs.options.rootPath, adminRouter)

// end admin js

// Redis cache enabled by env variable
if (process.env.USE_REDIS === 'true') {
    const getExpeditiousCache = require('express-expeditious')
    const cache = getExpeditiousCache({
        namespace: 'expresscache',
        defaultTtl: '1 minute',
        engine: require('expeditious-engine-redis')({
            redis: {
                host: process.env.REDIS_HOST,
                port: process.env.REDIS_PORT
            }
        })
    })
    app.use(cache)
}

// for parsing json
app.use(
        bodyParser.json({
            limit: '20mb',
            verify: (req, res, buf) => {
                req.rawBody = buf;
            },
        })
    )
    // for parsing application/x-www-form-urlencoded
app.use(
        bodyParser.urlencoded({
            limit: '20mb',
            extended: true
        })
    )
    // i18n
i18n.configure({
    locales: ['en', 'ar'],
    directory: `${__dirname}/locales`,
    defaultLocale: 'en',
    objectNotation: true
})
app.use(i18n.init)
    // init sentry
    // const Sentry = require("@sentry/node");
    // // or use es6 import statements
    // // import * as Sentry from '@sentry/node';

// const Tracing = require("@sentry/tracing");
// // or use es6 import statements
// // import * as Tracing from '@sentry/tracing';

// Sentry.init({
//     dsn: "https://dd87ed2f2d254441b6cabc587fa52c19@o1019753.ingest.sentry.io/5985613",

//     // Set tracesSampleRate to 1.0 to capture 100%
//     // of transactions for performance monitoring.
//     // We recommend adjusting this value in production
//     tracesSampleRate: 1.0,
// });

// const transaction = Sentry.startTransaction({
//     op: "test",
//     name: "My First Test Transaction",
// });

// setTimeout(() => {
//     try {
//         foo();
//     } catch (e) {
//         Sentry.captureException(e);
//     } finally {
//         transaction.finish();
//     }
// }, 99);
// Init all other stuff

app.use(cors())
app.use(passport.initialize())
app.use(compression())
app.use(helmet())
app.use(express.static('public'))
app.set('views', path.join(__dirname, 'views'))
app.engine('html', require('ejs').renderFile)
app.set('view engine', 'html')
app.use(require('./app/routes'))
app.listen(app.get('port'))


// Init MongoDB
initMongo()

module.exports = app // for testing